
from fastapi import FastAPI, HTTPException, BackgroundTasks
from pydantic import BaseModel
from crewai import Agent, Task, Crew
from typing import List, Optional
import logging
import asyncio
from functools import lru_cache
import os
from contextlib import asynccontextmanager

# Configure logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

# Initialize FastAPI app
app = FastAPI(title="CrewAI API Service")

class CrewRequest(BaseModel):
    objective: str
    context: Optional[str] = None
    max_iterations: Optional[int] = 3
    async_execution: Optional[bool] = True

class CrewResponse(BaseModel):
    task_id: str
    status: str
    result: Optional[str] = None

# In-memory store for task results
task_results = {}

@lru_cache()
def get_agents():
    """Cache and return agent instances to reduce initialization overhead"""
    researcher = Agent(
        role='Researcher',
        goal='Find and analyze relevant information',
        backstory='Expert at gathering and analyzing information',
        allow_delegation=False
    )
    
    writer = Agent(
        role='Writer',
        goal='Create clear and concise content',
        backstory='Expert at creating engaging content',
        allow_delegation=False
    )
    
    return [researcher, writer]

async def process_crew_task(task_id: str, crew_request: CrewRequest):
    """Execute crew task asynchronously"""
    try:
        # Initialize agents
        agents = get_agents()
        
        # Create tasks
        research_task = Task(
            description=f"Research: {crew_request.objective}",
            agent=agents[0]
        )
        
        writing_task = Task(
            description=f"Write content about: {crew_request.objective}",
            agent=agents[1]
        )
        
        # Initialize crew
        crew = Crew(
            agents=agents,
            tasks=[research_task, writing_task],
            max_iterations=crew_request.max_iterations
        )
        
        # Execute crew tasks
        result = await crew.kickoff()
        
        # Store result
        task_results[task_id] = {
            "status": "completed",
            "result": result
        }
        
    except Exception as e:
        logger.error(f"Error processing task {task_id}: {str(e)}")
        task_results[task_id] = {
            "status": "failed",
            "result": str(e)
        }

@app.post("/crew/tasks", response_model=CrewResponse)
async def create_crew_task(
    crew_request: CrewRequest,
    background_tasks: BackgroundTasks
):
    """Create and initiate a new crew task"""
    task_id = str(len(task_results) + 1)  # Simple ID generation
    
    if crew_request.async_execution:
        # Initialize task status
        task_results[task_id] = {
            "status": "processing",
            "result": None
        }
        
        # Schedule task execution
        background_tasks.add_task(
            process_crew_task,
            task_id,
            crew_request
        )
        
        return CrewResponse(
            task_id=task_id,
            status="processing"
        )
    else:
        # Synchronous execution
        await process_crew_task(task_id, crew_request)
        result = task_results[task_id]
        
        return CrewResponse(
            task_id=task_id,
            status=result["status"],
            result=result["result"]
        )

@app.get("/crew/tasks/{task_id}", response_model=CrewResponse)
async def get_task_status(task_id: str):
    """Get status and result of a specific task"""
    if task_id not in task_results:
        raise HTTPException(status_code=404, detail="Task not found")
    
    result = task_results[task_id]
    return CrewResponse(
        task_id=task_id,
        status=result["status"],
        result=result["result"]
    )

# Startup and shutdown events
@asynccontextmanager
async def lifespan(app: FastAPI):
    # Startup: Initialize any resources (e.g., connection pools)
    logger.info("Starting up CrewAI API Service")
    yield
    # Shutdown: Clean up resources
    logger.info("Shutting down CrewAI API Service")



from crewai.project import agent, crew, task, CrewBase
from cognition_core.crew import CognitionCoreCrewBase
from cognition_core.llm import init_portkey_llm
from crewai import Agent, Crew, Process, Task


@CrewBase
class Cognition(CognitionCoreCrewBase):
    """Cognition crew"""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    @agent
    def researcher(self) -> Agent:
        # Get raw config for LLM initialization
        raw_config = self.config_manager.get_config("agents")["researcher"]
        # Initialize LLM with config settings and portkey config
        llm = init_portkey_llm(
            model=raw_config["llm"],
            portkey_config=self.portkey_config,
        )
        # Pass file path to Agent for CrewAI's config loading
        agent = Agent(config=self.agents_config["researcher"], llm=llm, verbose=True)
        return agent

    @agent
    def reporting_analyst(self) -> Agent:
        # Get raw config for LLM initialization
        raw_config = self.config_manager.get_config("agents")["reporting_analyst"]
        # Initialize LLM with config settings and portkey config
        llm = init_portkey_llm(
            model=raw_config["llm"],
            portkey_config=self.portkey_config,
        )
        # Pass file path to Agent for CrewAI's config loading
        agent = Agent(
            config=self.agents_config["reporting_analyst"], llm=llm, verbose=True
        )
        return agent

    @task
    def research_task(self) -> Task:
        task = Task(config=self.tasks_config["research_task"])

        # Print all properties of the task
        # for key, value in vars(task).items():
        #     print(f"{key}: {value}")

        # exit(0)  # Remove this if you want the program to continue
        return task

    def search_web(self, query: str) -> str:
        return f"Searching the web for {query}"

    @task
    def reporting_task(self) -> Task:
        task = Task(config=self.tasks_config["reporting_task"], output_file="report.md")

        # Print all properties of the task
        # for key, value in vars(task).items():
        #     print(f"{key}: {value}")

        # exit(0)  # Remove this if you want the program to continue
        return task

    @crew
    def crew(self) -> Crew:
        """Creates the Cognition crew"""
        crew = Crew(
            agents=self.agents,
            tasks=self.tasks,
            process=Process.sequential,
            memory=True,
            verbose=True,
            short_term_memory=self.memory_service.get_short_term_memory(),
            entity_memory=self.memory_service.get_entity_memory(),
            long_term_memory=self.memory_service.get_long_term_memory(),
            embedder=self.memory_service.embedder,
        )
        # exit(0)
        return crew
